Package: stringr
Title: Simple, Consistent Wrappers for Common String Operations
Version: 1.6.0
Authors@R: c(
    person("Hadley", "Wickham", , "hadley@posit.co", role = c("aut", "cre", "cph")),
    person("Posit Software, PBC", role = c("cph", "fnd"))
  )
Description: A consistent, simple and easy to use set of wrappers around
    the fantastic 'stringi' package. All function and argument names (and
    positions) are consistent, all functions deal with "NA"'s and zero
    length vectors in the same way, and the output from one function is
    easy to feed into the input of another.
License: MIT + file LICENSE
URL: https://stringr.tidyverse.org,
        https://github.com/tidyverse/stringr
BugReports: https://github.com/tidyverse/stringr/issues
Depends: R (>= 4.1.0)
Imports: cli, glue (>= 1.6.1), lifecycle (>= 1.0.3), magrittr, rlang
        (>= 1.0.0), stringi (>= 1.5.3), vctrs (>= 0.4.0)
Suggests: covr, dplyr, gt, htmltools, htmlwidgets, knitr, rmarkdown,
        testthat (>= 3.0.0), tibble
VignetteBuilder: knitr
Config/Needs/website: tidyverse/tidytemplate
Config/potools/style: explicit
Config/testthat/edition: 3
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2025-11-03 22:09:58 UTC; hadleywickham
Author: Hadley Wickham [aut, cre, cph],
  Posit Software, PBC [cph, fnd]
Maintainer: Hadley Wickham <hadley@posit.co>
Repository: CRAN
Date/Publication: 2025-11-04 14:00:02 UTC
Built: R 4.6.0; ; 2026-04-25 11:14:29 UTC; unix
