# Generated by roxygen2 (4.1.0): do not edit by hand

export("%>%")
export("str_sub<-")
export(boundary)
export(coll)
export(fixed)
export(ignore.case)
export(invert_match)
export(perl)
export(regex)
export(str_c)
export(str_conv)
export(str_count)
export(str_detect)
export(str_dup)
export(str_extract)
export(str_extract_all)
export(str_join)
export(str_length)
export(str_locate)
export(str_locate_all)
export(str_match)
export(str_match_all)
export(str_order)
export(str_pad)
export(str_replace)
export(str_replace_all)
export(str_replace_na)
export(str_sort)
export(str_split)
export(str_split_fixed)
export(str_sub)
export(str_subset)
export(str_to_lower)
export(str_to_title)
export(str_to_upper)
export(str_trim)
export(str_wrap)
export(word)
import(stringi)
importFrom(magrittr,"%>%")
