% Generated by roxygen2 (4.1.0): do not edit by hand
% Please edit documentation in R/stringr.R
\name{stringr}
\alias{stringr}
\title{Fast and friendly string manipulation.}
\description{
Fast and friendly string manipulation.
}

